/*

 ROTATING PULSING STARS - Roald Baudoux 2011
 EACH STAR PULSES AT A DIFFERENT RATE AND ROTATES IN 3 D AROUND ITS CENTER
 THE GLOBAL 3D SPACE ROTATES AROUND ITSELF TOO
 HSB COLORS WITH VARYING HUE
 
*/

import processing.opengl.*;

// VARIABLES
int numCubes = 200;
int maxDistance = 400;
PVector[] places = new PVector[numCubes]; // stores positions of cubes
PVector[] angles = new PVector[numCubes]; // stores orientations of cubes
PVector[] rotFreq = new PVector[numCubes]; // stores rotation frequencies of cubes in 3 D
float[] pulsFreq = new float[numCubes]; // stores pulsation frequencies for hue+saturation variation
float phaseX ; // phase for rotation around own center 
float phaseY ;
float phaseZ ;
int rate = 120; // set frameRate here
float currentPulse; //pulsation for HSB color variation
float currentHue;
float currentSat;
float myHue1 = 40; // star's start hue
float myHue2 = 180; // star's final hue
float mySat1 = 255; // star's start saturation
float mySat2 = 0; // star's intermediate saturation
float mySat3 = 255; // star's final saturation
float xRotateFrequency =  .503 ; //frequency for rotation around x axis
float yRotateFrequency =  .307 ; //frequency for rotation around y axis
float zRotateFrequency = .409 ; //frequency for rotation around z axis

void setup() {
  size (500, 500, OPENGL);
  colorMode(HSB);
  frameRate(rate);
  //lights();
  noStroke();
  //stroke(44, 255, 255, 255);
  //strokeWeight(0);
  for (int i = 0; i < numCubes; i++) {
    places[i] = new PVector (random(0, maxDistance), random(0, maxDistance), random(-100, maxDistance)) ;
    angles[i] = new PVector (random(0, TWO_PI), random(0, TWO_PI), random(0, TWO_PI)) ;
    pulsFreq[i] = pow(10., random(-1.0, 0.5)) ;
    rotFreq[i] = new PVector (pow(10., random(-1., 0.001)), pow(10., random(-1, 0.001)), pow(10., random(-1, 0.001))) ;
  }
}

void draw() {
  background (0, 0, 0, 0);
  smooth();
  translate(width/2, height/2, 0);
  rotateX(xRotateFrequency * float(frameCount) / float(rate));
  rotateY(yRotateFrequency * float(frameCount) / float(rate));
  rotateZ(zRotateFrequency * float(frameCount) / float(rate));
  for (int i = 0; i < numCubes; i++) {
    pushMatrix() ;
    smooth();
    // POSITION AROUND CENTER
    rotateX(angles[i].x);
    rotateY(angles[i].y);
    rotateZ(angles[i].z);
    translate(places[i].x, places[i].y, places[i].z);
    // ORIENTATION
    phaseX = (((rotFreq[i].x * float(frameCount) / float(rate)) * TWO_PI) % TWO_PI);
    phaseY = (((rotFreq[i].y * float(frameCount) / float(rate)) * TWO_PI) % TWO_PI);
    phaseZ = (((rotFreq[i].z * float(frameCount) / float(rate)) * TWO_PI) % TWO_PI);
    rotateX(phaseX);
    rotateY(phaseY);
    rotateZ(phaseZ);
    // COLOR
    currentPulse = unipolarTriangle((pulsFreq[i] * float(frameCount) / float(rate)) % 1.) ;
    // SETS HUE
    if (currentPulse < 0.5 ) {
      currentHue = myHue1 ;
    } else {
      currentHue = myHue2 ;     
    }
    // SETS SATURATION
    if (currentPulse < 0.5 ) {
      currentSat = int(((1. - (2. * currentPulse)) * mySat1) + ((2. * currentPulse) * mySat2)) ;
    } else {
      currentSat = int(((1. - (2. * (currentPulse - 0.5))) * mySat2) + ((2. * (currentPulse - 0.5)) * mySat3)) ;      
    }
    fill(currentHue, currentSat, 255, 255);
    scale(30);
    // DRAWS STAR
    beginShape() ;
    vertex(sin(0), cos(0), 0);
    vertex((sin(TWO_PI/10))/2.6, (cos(TWO_PI/10))/2.6, 0);
    vertex(sin(2 * TWO_PI/10), cos(2 * TWO_PI/10), 0);
    vertex((sin(3 * TWO_PI/10))/2.6, (cos(3 * TWO_PI/10))/2.6, 0);
    vertex(sin(4 * TWO_PI/10), cos(4 * TWO_PI/10), 0);
    vertex((sin(5 * TWO_PI/10))/2.6, (cos(5 * TWO_PI/10))/2.6, 0);
    vertex(sin(6 * TWO_PI/10), cos(6 * TWO_PI/10), 0);
    vertex((sin(7 * TWO_PI/10))/2.6, (cos(7 * TWO_PI/10))/2.6, 0);
    vertex(sin(8 * TWO_PI/10), cos(8 * TWO_PI/10), 0);
    vertex((sin(9 * TWO_PI/10))/2.6, (cos(9 * TWO_PI/10))/2.6, 0);
    endShape(CLOSE);
    popMatrix() ;
  }

}

float unipolarTriangle(float phase) 
{
 float amp;
 amp = min(phase, 1.) ;
 if (amp > 0.5 ) {
   amp = 0.5 - (amp - 0.5);
 }
 amp *= 2.;
 return amp;
}

